#!/bin/bash
# --------------------------------------------------------------------------
# This script sets the right fan controller.
# It will start at 33.5% of full speed (RPM) when the remote temp sensor (Q13)
# reaches 44C.
# The spead will gradually ramp as temperature increases, with max speed
# (100%) reached at a temp of 105C.
#
# This is only temporary until the HWM can take over fan control. 
#----------------------------------------------------------------------------

echo ""
echo "Running ${0##*/}"

i2cset -y 1 0x19 0x00 0xdd # Only remote temp sensor used
i2cset -y 1 0x19 0x01 0x3f # Tach mode inverted (low-side control)
i2cset -y 1 0x19 0x21 0x55 # Duty cycle is 33% when temp is 44C
i2cset -y 1 0x19 0x1c 0x2c # Passive cooling temp threshold is 44C
i2cset -y 1 0x19 0x24 0x2c # Local temp thresholds and slopes
i2cset -y 1 0x19 0x25 0x2c # Remote temp thresholds and slopes
i2cset -y 1 0x19 0x23 0xd2 # Ramp enable, slope definitions.

echo " - determine if left-side fan controller exists."
echo " - this is the id of the controller if it does."
#this also seems to fix the problem where left side fan suddenly spins to 100% when HWM takes over.
i2cget -y 1 0x18 0x3d
if [[ $? -eq 0 ]]; then
	echo " -- left side fan exists. "
	i2cset -y 1 0x18 0x00 0xdd # Only remote temp sensor used
	i2cset -y 1 0x18 0x01 0x3f # Tach mode inverted (low-side control)
	i2cset -y 1 0x18 0x21 0x55 # Duty cycle is 33% when temp is 44C
	i2cset -y 1 0x18 0x1c 0x2c # Passive cooling temp threshold is 44C
	i2cset -y 1 0x18 0x24 0x2c # Local temp thresholds and slopes
	i2cset -y 1 0x18 0x25 0x2c # Remote temp thresholds and slopes
	i2cset -y 1 0x18 0x23 0xd2 # Ramp enable, slope definitions.
else
	echo " -- left side fan does not exist. "
fi


echo " -- ${0##*/} finished successfully"
