import logging
import json
import os
import metersConstants
try:
    import netifaces
except ImportError as ex:
    print "Importing fake netifaces"
    from mock import netifaces

logger = logging.getLogger("__meters__")


def preset_labels():
    """  Getting labels of the last applied/displayed preset

    :return: List of labels
    """
    labels = ['Input ' + str(idx) for idx in range(1, 17)]
    preset_name = 'Preset 1'
    preset_id = 1
    try:
        if os.path.isfile("/run/shm/database/labels_preset.json"):
            with open('/run/shm/database/labels_preset.json') as labels_json_file:
                label_per_preset = json.load(labels_json_file)

            if label_per_preset:
                preset_name = label_per_preset[0]["preset_name"]
                labels = label_per_preset[1]["preset_labels"]
                preset_id = label_per_preset[2]["preset_id"]

    except Exception as ex:
        logger.debug("Error on create_json_file : %s", str(ex))

    return preset_name, preset_id, labels


def product_name():

    iam_product_name = "iAM-Series"
    try:
        if os.path.isfile(metersConstants.DB_SHM_PATH + "/iam_product.json"):
            with open(metersConstants.DB_SHM_PATH + '/iam_product.json') as product_json_file:
                iam_product_name = json.load(product_json_file)
    except Exception as ex:
        logger.debug("Error on create_json_file : %s", str(ex))
        return "iAM-Series"

    return iam_product_name


def create_json_file(file_name, file_path, data_list):
    """

    :param file_name: JSON file name
    :param file_path: File Location
    :param data_list: Content of JSON File
    :return: Boolean : True Success else False
    """
    try:
        with open(file_path + file_name, 'w') as fp:
            json.dump(data_list,  fp, separators=(',', ':'))
    except Exception as ex :
        logger.info("ERROR : Creating json file.Error : " + str (ex) )
        return False

    return True


def get_ip_by_interface(net_interface):
    avahi_ip_address = None
    try:
        ifaces = netifaces.interfaces()
        for iface in ifaces:
            if iface == net_interface:
                ip_addrs = netifaces.ifaddresses(iface)
                if netifaces.AF_INET in ip_addrs:
                    ip_addr_desc = ip_addrs[netifaces.AF_INET]
                    ip_addr_desc = ip_addr_desc[0]
                    avahi_ip_address = ip_addr_desc['addr']
    except Exception as ex:
        logger.exception("Error: %s", str(ex))
        return None
    return avahi_ip_address
