import zmq
import json
import logging
import metersConstants
import metersHelper
from cffi import FFI
import requests
import time
from socketIO_client import SocketIO, BaseNamespace
from datetime import datetime

logging.basicConfig()
logger = logging.getLogger("__meters_sub__")

lu_pref_version = 0

is_meter_connected = False
class HwmMetersDataNamespace(BaseNamespace):

    def on_connect(self):
        global is_meter_connected
        is_meter_connected = True
        print('[MeterData Connected..!]')


    def on_reconnect(self):
        global is_meter_connected
        is_meter_connected = True
        print('[MeterData Reconnected]')

    def on_disconnect(self):
        global is_meter_connected
        is_meter_connected = False
        print('[MeterData Disconnected]')

is_lu_connected = False
class HwmLuDataNamespace(BaseNamespace):

    def on_connect(self):
        global is_lu_connected

        is_lu_connected = True
        print('[LuData Connected..!]')


    def on_reconnect(self):
        global is_lu_connected
        is_lu_connected = True
        print('[LuData Reconnected]')

    def on_disconnect(self):
        global is_lu_connected
        is_lu_connected = False
        print('[LuData Disconnected]')

def send_meter_data(data):

    try:
        global hwmMetersDataNamespace
        global is_meter_connected
        if hwmMetersDataNamespace is not None and is_meter_connected:
            # logger.warn(data['meter_data'][5]['Loudness']['LuData']['Lu'])
            # bgn_emit = datetime.now()
            # logger.warn("bgn emit: %s", str())
            hwmMetersDataNamespace.emit('hwmMetersDataEvent', data, broadcast=False)
            # end_emit = datetime.now()
            # logger.warn("emit took: %s", str(end_emit - bgn_emit))
    except Exception as e:
        logger.exception("send_meter_data().Error :" + str(e).replace('"', ''))


def send_lu_data(data):

    try:
        global hwmLuDataNamespace
        global is_lu_connected
        if hwmLuDataNamespace is not None and is_lu_connected:
            # logger.warn(data['meter_data'][5]['Loudness']['LuData']['Lu'])
            # bgn_emit = datetime.now()
            # logger.warn("bgn emit: %s", str())
            hwmLuDataNamespace.emit('hwmLuDataEvent', data, broadcast=False)
            # end_emit = datetime.now()
            # logger.warn("emit took: %s", str(end_emit - bgn_emit))

    except Exception as e:
        logger.exception("send_lu_data().Error :" + str(e).replace('"', ''))


def loudness_value(bin_value, divisor=10.0):
    try:
        buffer_data = ffi.from_buffer(bin_value)
        tmp_data = ffi.cast('Loudness_t *', buffer_data)
        return round(float(tmp_data.IntValue / divisor), 2)
    except Exception as excptn:
        pass

    return 0.0


def meter_data_msg(socket):
    """Getting METER Data Messages(SUB)
    :param socket: string HWM Socket
    :return:
    """

    try:
        data = socket.recv_multipart()
        topic_received = data[0]
        if topic_received == metersConstants.METER_DATA_TOPIC:
            try:
                step_in_loop = 1
                buffer_peak = ffi.from_buffer(data[1])
                buffer_avg = ffi.from_buffer(data[2])
                buffer_phase = ffi.from_buffer(data[3])
                avgs = ffi.cast('avg_meter_data_t *', buffer_avg)
                peaks = ffi.cast('peak_meter_data_t *', buffer_peak)
                phases = ffi.cast('phase_indicator_t *', buffer_phase)

                avg_list = [(float(avgs.average[idx]) / 100) if avgs.average[idx] >= -7000 else -70.00
                            for idx in range(0, 16, step_in_loop)]
                peak_list = [(float(peaks.peak[idx]) / 100) if peaks.peak[idx] >= -7000 else -70.00
                             for idx in range(0, 16, step_in_loop)]
                phase_list = [1 for idx in range(0, 16)]

                if 0 < phases.PhaseIndicator < 256:
                    logger.debug("phases.PhaseIndicator: %s :  %s", str(phases.PhaseIndicator),
                                format(phases.PhaseIndicator, '#010b'))
                    phase_list[0] = phase_list[1] = 0 if phases.PhaseIndicator & 1 != 0 else 1
                    for idx in range(1, 8, step_in_loop):
                        if phases.PhaseIndicator & (1 << idx) != 0:
                            phase_list[(idx * 2)] = phase_list[(idx * 2) + 1] = 0

                global meter_info_count
                global loudness_json_data
                global loudness_preferences
                global lu_pref_version
                meter_data = {'meter_data': [{'avg': avg_list},
                                             {'peak': peak_list},
                                             {'xLabels': preset_labels},
                                             {'count': meter_info_count},
                                             {'preset_name': cur_meter_obj.name},
                                             {'Prg': loudness_preferences["LuPref"]["ClId"]}
                                             #{'Loudness': loudness_json_data},
                                             #{'LoudnessPref': loudness_preferences, 'LuPrefVersion': lu_pref_version},
                                             #{'phases': phase_list}
                                             ]}


                #meter_data = {'meter_data' : "Test meter data"}

                send_meter_data(meter_data)
                #millis = int(round(time.time() * 1000))
                #print millis
                #meter_info_count = 1

            except Exception as sockErr:
                logger.exception("HWM message : %s", str(sockErr))
                logger.exception(data)
    except zmq.ZMQError as zmqErr:
        print "ZMQ error"
        logger.exception("ZMQ Error : " + str(zmqErr))


def loudness_data_msg(socket):
    """

    :param socket:
    :return:
    """
    try:
        loudness_data = socket.recv_multipart()
        topic = loudness_data[0]
        global loudness_preferences
        global lu_pref_version
        global loudness_json_data
        if topic == metersConstants.LOUDNESS_DATA_TOPIC:
            try:

                result = [loudness_value(loudness_data[idx], 10.0 if idx < 7 else 100.0) for idx in
                          range(1, len(loudness_data))]
                l_display_value = result[1]
                l_window = "Momentary"
                l_display_text = "LU-M"
                loudness_window = loudness_preferences["LuPref"]["LuW"]
                if loudness_window == "Short Term":
                    l_display_value = result[2]
                    l_window = "Short Term"
                    l_display_text = "LU-ST"
                elif loudness_window == "Custom":
                    l_display_value = result[3]
                    l_window = "Integrated"
                    l_display_text = "LU-C"

                l_reference = loudness_preferences["LuPref"]["RefL"]
                l_tolerance = loudness_preferences["LuPref"]["AlT"]

                max_value = float(l_reference) + float(l_tolerance)
                min_value = float(l_reference) - float(l_tolerance)
                l_alarm = loudness_preferences["LuPref"]["Alarm"]
                color = "lu-bg-default"
                if l_alarm == "Over":
                    color = "lu-bg-over" if max_value < l_display_value else "lu-bg-valid"
                elif l_alarm == "Under":
                    color = "lu-bg-under" if min_value > l_display_value else "lu-bg-valid"
                elif l_alarm == "Both":
                    if max_value < l_display_value:
                        color = "lu-bg-over"
                    elif min_value > l_display_value:
                        color = "lu-bg-under"
                    else:
                        color = "lu-bg-valid"
                elif l_alarm == "Off":
                    color = "lu-bg-off"

                loudness_json_data = {'LuData': {'MaxM': result[0],
                                                       #'Momentary': result[1],
                                                       #'ShortTerm': result[2],
                                                       #'Cust': result[3],
                                                       'Lra': result[4],
                                                       'Prog': result[5],
                                                       'TP': result[6],
                                                       #'TruePeak': result[7],
                                                       'Color': color,
                                                       'Lu': l_display_value,
                                                       'LuW': l_window,
                                                       'xTick': l_display_text
                                                       }
                                      }


                #global meter_info_count
                #global loudness_json_data
                #global loudness_preferences
                #global lu_pref_version
                lu_data = {'lu_data': [
                                             {'Lu': loudness_json_data},
                                             {'LuPref': loudness_preferences, 'LuPrefV': lu_pref_version},

                                             ]}

                #lu_data = {'lu_data' : "Test lu data"}

                send_lu_data(lu_data)
                #millis = int(round(time.time() * 1000))
                #print millis

            except Exception as excptn:
                logger.exception("Error %s", str(excptn))
        elif topic == metersConstants.LOUDNESS_CLUSTER_TOPIC:
            try:
                lu_pref_version += 1
                fp_loudness_cluster = json.loads(loudness_data[1])
                if "ClusterId" in fp_loudness_cluster:
                    loudness_preferences["LuPref"]["ClId"] = "{0:0>2}".format(
                        str(fp_loudness_cluster["ClusterId"]))
                    global meter_info_count
                    meter_info_count = meter_info_count + 1
            except Exception as excptn:
                logger.exception("Error %s", str(excptn))
        elif topic == metersConstants.LOUDNESS_CONF_TOPIC:
            try:
                lu_pref_version += 1
                fp_loudness_conf = json.loads(loudness_data[1])
                if "LoudnessWindow" in fp_loudness_conf:
                    loudness_preferences["LuPref"]["LuW"] = fp_loudness_conf["LoudnessWindow"]
                if "LoudnessWindowTime" in fp_loudness_conf:
                    loudness_preferences["LuPref"]["LuWT"] = fp_loudness_conf["LoudnessWindowTime"]
                if "LraWindow" in fp_loudness_conf:
                    loudness_preferences["LuPref"]["LraW"] = fp_loudness_conf["LraWindow"]
                if "Mode" in fp_loudness_conf:
                    loudness_preferences["LuPref"]["Mode"] = fp_loudness_conf["Mode"]
            except Exception as excptn:
                logger.exception("Error %s", str(excptn))
        # elif topic == metersConstants.LOUDNESS_COMMAND_TOPIC:
        #     logger.warning("topic %s", topic)
        #     logger.warning("loudness_data")
        #     logger.warning(loudness_data)
        # elif topic == metersConstants.LOUDNESS_DSP_DATA_TOPIC:
        #     logger.warning("topic %s", topic)
        #     logger.warning("loudness_data")
        #     logger.warning(loudness_data)
    except zmq.ZMQError as zmqErr:
        print "ZMQ error"
        logger.exception("ZMQ Error : " + str(zmqErr))


def set_loudness_msg(topic, message):
    """

    :param socket:
    :return:
    """

    if topic == metersConstants.SET_LOUDNESS_CONF_TOPIC:
        try:
            pref_info = json.loads(message)
            global loudness_preferences
            preferences = {}
            preferences["ClId"] = "{0:0>2}".format(str(pref_info["ClusterId"])) if "ClusterId" in pref_info else "01"
            preferences["RefL"] = pref_info["ReferenceLevel"] if "ReferenceLevel" in pref_info else -24
            loudness_window = preferences["LuW"] = pref_info["LoudnessWindow"] if "LoudnessWindow" in pref_info else "Momentary"

            loudness_standard = preferences["LuSt"] = pref_info["LoudnessStandard"] if "LoudnessStandard" in pref_info else "ITU BS1770-3"
            if loudness_standard == "EBU R128 (+9)":
                preferences["Unit"] = "LUFS"
            elif loudness_standard == "EBU R128 (+18)":
                preferences["Unit"] = "LUFS"
            else:
                preferences["Unit"] = "LKFS"
            preferences["AlT"] = pref_info["AlarmTolerance"] if "AlarmTolerance" in pref_info else 2
            preferences["Alarm"] = pref_info["Alarm"] if "Alarm" in pref_info else "Both"
            preferences["Mode"] = pref_info["Mode"] if "Mode" in pref_info else "Continuous"
            loudness_alarm = pref_info["Alarm"] if "Alarm" in pref_info else "Off"

            loudness_preferences = {"LuPref": preferences}
        except Exception as excptn:
            logger.exception("Error %s", str(excptn))


ffi = FFI()
ffi.cdef("""

 typedef struct
 {
    int16_t peak[16];
 }peak_meter_data_t;

 typedef struct
 {
    int16_t average[16];
 }   avg_meter_data_t;

typedef struct
 {
     int32_t PhaseIndicator;
 }   phase_indicator_t;


// Generic struct for all values of loudness data.

typedef struct
{
   int16_t IntValue;

} Loudness_t;

""")


lib = ffi.dlopen(None)
logger.debug('Loaded lib {0}'.format(lib))

ipv4ll_mb_ip = metersHelper.get_ip_by_interface('eth0')
METER_SOCKET = "tcp://{ip_addr}:{port_number}".format(ip_addr=ipv4ll_mb_ip if ipv4ll_mb_ip else "0.0.0.0",
                                                      port_number=metersConstants.METER_SOCKET_PORT)

WEB_SOCKET = "tcp://{ip_addr}:{port_number}".format(ip_addr=ipv4ll_mb_ip if ipv4ll_mb_ip else "0.0.0.0",
                                                    port_number=metersConstants.WEB_SOCKET_PORT)

LOUDNESS_SOCKET = "tcp://{ip_addr}:{port_number}".format(ip_addr=ipv4ll_mb_ip if ipv4ll_mb_ip else "0.0.0.0",
                                                         port_number=metersConstants.LOUDNESS_SOCKET_PORT)

logger.warning("LOUDNESS_SOCKET : %s", LOUDNESS_SOCKET)
logger.warning("WEB_SOCKET : %s", WEB_SOCKET)

meter_context = zmq.Context()
meter_socket = meter_context.socket(zmq.SUB)
meter_socket.connect(METER_SOCKET)

web_context = zmq.Context()
web_socket = web_context.socket(zmq.SUB)
web_socket.connect(WEB_SOCKET)

loudness_context = zmq.Context()
loudness_socket = loudness_context.socket(zmq.SUB)
loudness_socket.connect(LOUDNESS_SOCKET)


class MeterSet:
    def __init__(self, name="Preset 1", channels=None, speakers=None):
        self.name = name
        self.channels = channels
        self.speakers = speakers

    def __str__(self):
        return str(self.__dict__)

    def __eq__(self, other):
        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not self == other


cur_meter_obj = MeterSet("Preset 1", "[1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16]",
                         "['L','L','L','L','L','L','L','L','L','L','L','L','L','L','L','L']")

# avg_list = []

#
# Create a poller and register the events we want to poll
#
#time.sleep(10)
poller = zmq.Poller()
poller.register(meter_socket, zmq.POLLIN | zmq.POLLOUT)
poller.register(web_socket, zmq.POLLIN | zmq.POLLOUT)
poller.register(loudness_socket, zmq.POLLIN | zmq.POLLOUT)


#
# Topics
#
meter_socket.setsockopt(zmq.SUBSCRIBE, metersConstants.METER_DATA_TOPIC)
web_socket.setsockopt(zmq.SUBSCRIBE, metersConstants.PRESET_CONFIGURATION_TOPIC)
web_socket.setsockopt(zmq.SUBSCRIBE, metersConstants.PRESET_RECALL_TOPIC)
web_socket.setsockopt(zmq.SUBSCRIBE, metersConstants.SET_LOUDNESS_CONF_TOPIC)


loudness_socket.setsockopt(zmq.SUBSCRIBE, metersConstants.LOUDNESS_CLUSTER_TOPIC)
loudness_socket.setsockopt(zmq.SUBSCRIBE, metersConstants.LOUDNESS_COMMAND_TOPIC)
loudness_socket.setsockopt(zmq.SUBSCRIBE, metersConstants.LOUDNESS_CONF_TOPIC)
loudness_socket.setsockopt(zmq.SUBSCRIBE, metersConstants.LOUDNESS_DATA_TOPIC)
loudness_socket.setsockopt(zmq.SUBSCRIBE, metersConstants.LOUDNESS_DSP_DATA_TOPIC)

# Is meter app ready to receive data?
# Wait for 60 secs.
app_url = metersConstants.URL_METER_APP
for idx in range(0, 60):
    try:
        response = requests.get(app_url)
        break
    except requests.ConnectionError:
        pass
    except Exception as excptn:
        pass
    time.sleep(1)

tmp_channels = [idx for idx in range(1, 17)]
tmp_speakers = ['S' for idx in range(1, 17)]
preset_labels = ['I{0:0>2}'.format(str(idx)) for idx in range(1, 17)]
meter_info_count = 1

socketIO = SocketIO('0.0.0.0', 5000)



hwmMetersDataNamespace = socketIO.define(HwmMetersDataNamespace, '/hwmMetersData')

hwmLuDataNamespace = socketIO.define(HwmLuDataNamespace, '/hwmLuData')


# Default
loudness_json_data = {'LuData': {'MaxM': -99.0,
                                       'Lra': 0.0,
                                       'Prog': -99.0,
                                       'Cust': -99.0,
                                       'TP': -99.0,
                                       'LuW': 'Momentary',
                                       'Color': 'lu-bg-default',
                                       'Lu': -99.0,
                                       'xTick': 'LU-M'
                                       }
                      }

loudness_preferences = {'LuPref': {'ClId': '01',
                                             'RefL': -24,
                                             'LuW': 'Momentary',
                                             'Alarm': 'Both',
                                             'LuSt': 'ITU BS1770-3',
                                             'AlT': 2,
                                             'Unit': 'LKFS',
                                             'LuWT': 3,
                                             'LraW': 9,
                                             }
                        }
while True:
    socks = dict(poller.poll(100))
    # Get all sockets that can do something
    try:
        if meter_socket in socks and socks[meter_socket] == zmq.POLLIN:
            meter_data_msg(meter_socket)

        elif web_socket in socks and socks[web_socket] == zmq.POLLIN:
            data = web_socket.recv_multipart()
            topic_received = data[0]
            if topic_received != "PresetConfiguration":
                set_loudness_msg(topic_received, data[1])
            else:
                preset_configuration = json.loads(data[1])
                if preset_configuration and 'Name' in preset_configuration and 'Preset' in preset_configuration['Name']:
                    tmp_channels2 = [idx for idx in range(1, 17)]
                    preset_labels = ['In{0:0>2}'.format(str(idx)) for idx in range(1, 17)]
                    preset_name = preset_configuration['Name']['Preset']
                    if 'Clusters' in preset_configuration and preset_configuration['Clusters']:
                        tmp_clusters = preset_configuration['Clusters']
                        for cluster_id in range(len(tmp_clusters)):
                            cluster = preset_configuration['Clusters'][cluster_id]
                            meter_bgn = cluster["FirstMeter"] - 1
                            meter_end = cluster["NumberOfMeters"] + meter_bgn
                            for meter in range(meter_bgn, meter_end):
                                tmp_name = str(preset_labels[meter]) + "-Cl{0:0>2}".format(str(cluster_id+1))
                                preset_labels[meter] = tmp_name
                    tmp_meter_obj = MeterSet(preset_name, str(tmp_channels), str(tmp_speakers))
                    if tmp_meter_obj != cur_meter_obj:
                        cur_meter_obj = tmp_meter_obj
                        meter_info_count = meter_info_count + 1

        elif loudness_socket in socks and socks[loudness_socket] == zmq.POLLIN:
            loudness_data_msg(loudness_socket)

    except Exception as e:
        logger.exception("set_input_src.()Error :" + str(e).replace('"', ''))
