import os
import sys
import logging

from flask import Flask
from flask_sqlalchemy import SQLAlchemy
from flask_login import LoginManager
from flask.ext.cache import Cache
BASE_LOGLEVEL = os.environ.get('PYTHON_LOGLEVEL', 'WARN')
#BASE_LOGLEVEL = os.environ.get('PYTHON_LOGLEVEL', 'DEBUG')
logging.basicConfig(level=BASE_LOGLEVEL)
logger = logging.getLogger(__name__)
basedir = os.path.abspath(os.path.dirname(__file__))

app = Flask(__name__)
app.debug = int(os.environ.get('FLASK_DEBUG', 0))
app.config.from_object('appconfig')
if app.debug:
    logger.warn("FLASK_DEBUG set, running in debug mode")
app.logger.setLevel(BASE_LOGLEVEL)

db = SQLAlchemy(app)
#configure authentication
login_manager = LoginManager()
login_manager.session_protection = "strong"
login_manager.login_view = "login"
login_manager.init_app(app)

# Other configuration
cache = Cache(app, config={
    'CACHE_TYPE': 'simple',
})

try:
    if int(os.environ.get('LOAD_DEBUG_TOOLBAR', 0)):
        app.debug = True
        logger.warn("Loading DebugToolbarExtension; forcing app.debug=True")
        from flask_debugtoolbar import DebugToolbarExtension
        # Specify the debug panels you want
        app.config['DEBUG_TB_PANELS'] = [
            'flask_debugtoolbar.panels.versions.VersionDebugPanel',
            'flask_debugtoolbar.panels.timer.TimerDebugPanel',
            'flask_debugtoolbar.panels.headers.HeaderDebugPanel',
            'flask_debugtoolbar.panels.request_vars.RequestVarsDebugPanel',
            'flask_debugtoolbar.panels.template.TemplateDebugPanel',
            'flask_debugtoolbar.panels.sqlalchemy.SQLAlchemyDebugPanel',
            'flask_debugtoolbar.panels.logger.LoggingPanel',
            'flask_debugtoolbar.panels.profiler.ProfilerDebugPanel',
            # Add the line profiling
            'flask_debugtoolbar_lineprofilerpanel.panels.LineProfilerPanel'
        ]
        toolbar = DebugToolbarExtension(app)
    else:
        pass
except ImportError:
    logger.warn("ImportError: DebugToolbarExtension")
    pass

# Configure blueprints
from blueprints.meters import meters

app.register_blueprint(meters)
