"""Init audiodb and data migration

Revision ID: 3982e06eeb8c
Revises: 7277acac375f
Create Date: 2018-01-10 06:22:21.041442

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = '3982e06eeb8c'
down_revision = None
branch_labels = None
depends_on = None

def create_table_GroupUser():
    try:
        op.create_table('GroupUser',
        sa.Column('id', sa.Integer(), nullable=False),
        sa.Column('name', sa.String(length=100), nullable=False),
        sa.Column('active', sa.String(length=10), nullable=False),
        sa.Column('last_Updated_date', sa.DateTime(), nullable=False),
        sa.PrimaryKeyConstraint('id')
        )
    except:
        print "Table GroupUser already exists."

def create_table_Log():
    try:
        op.create_table('Log',
        sa.Column('id', sa.Integer(), nullable=False),
        sa.Column('error_type', sa.Integer(), nullable=False),
        sa.Column('status', sa.String(length=100), nullable=False),
        sa.Column('program_name', sa.String(length=100), nullable=False),
        sa.Column('created_date', sa.DateTime(), nullable=False),
        sa.PrimaryKeyConstraint('id', 'error_type')
        )
    except:
        print "Table MailSetup already exists."


def create_table_MessageController():
    try:
        op.create_table('MessageController',
        sa.Column('id', sa.Integer(), nullable=False),
        sa.Column('item', sa.String(length=100), nullable=False),
        sa.Column('parameter_value_1', sa.String(length=100), nullable=True),
        sa.Column('parameter_value_2', sa.String(length=100), nullable=True),
        sa.Column('is_sent', sa.String(length=1), nullable=True),
        sa.Column('last_Updated_date', sa.DateTime(), nullable=True),
        sa.Column('sent_date', sa.DateTime(), nullable=True),
        sa.PrimaryKeyConstraint('id'),
        sa.UniqueConstraint('item')
        )
    except:
        print "Table MessageController already exists."


def create_table_Software():
    try:
        op.create_table('Software',
        sa.Column('id', sa.Integer(), nullable=False),
        sa.Column('name', sa.String(length=100), nullable=False),
        sa.Column('version', sa.String(length=100), nullable=False),
        sa.Column('release', sa.String(length=100), nullable=True),
        sa.Column('release_date', sa.DateTime(), nullable=True),
        sa.Column('description', sa.String(length=100), nullable=True),
        sa.Column('last_Updated_date', sa.DateTime(), nullable=False),
        sa.PrimaryKeyConstraint('id')
        )
    except:
        print "Table Software already exists."

def create_table_Unit():
    try:
        op.create_table('Unit',
        sa.Column('id', sa.Integer(), nullable=False),
        sa.Column('name', sa.String(length=100), nullable=False),
        sa.PrimaryKeyConstraint('id')
        )
    except:
        print "Table Unit already exists."

def create_table_User():
    try:
        op.create_table('User',
        sa.Column('id', sa.Integer(), nullable=False),
        sa.Column('username', sa.String(length=120), nullable=True),
        sa.Column('email', sa.String(length=120), nullable=True),
        sa.Column('password_hash', sa.String(), nullable=False),
        sa.Column('firstname', sa.String(length=120), nullable=False),
        sa.Column('lastname', sa.String(length=120), nullable=False),
        sa.Column('user_group_id', sa.Integer(), nullable=False),
        sa.Column('last_Updated', sa.DateTime(), nullable=False),
        sa.Column('last_Updated_by_user', sa.String(length=120), nullable=False),
        sa.ForeignKeyConstraint(['user_group_id'], ['GroupUser.id'], ),
        sa.PrimaryKeyConstraint('id'),
        sa.UniqueConstraint('email'),
        sa.UniqueConstraint('username')
        )
    except:
        print "Table User already exists."

def create_table_AudioPreference():
    try:
        op.create_table('AudioPreference',
        sa.Column('id', sa.Integer(), nullable=False),
        sa.Column('meter_scale', sa.String(length=50), nullable=False),
        sa.Column('solos_mutes', sa.String(length=50), nullable=False),
        sa.Column('mute_speakers', sa.String(length=50), nullable=False),
        sa.Column('xlr_out_affected_by', sa.String(length=50), nullable=False),
        sa.Column('xlr_out_mute_solos', sa.String(length=50), nullable=False),
        sa.Column('adjust_trim_out', sa.String(length=50), nullable=False),
        sa.Column('last_Updated_date', sa.DateTime(), nullable=False),
        sa.Column('last_Updated_by_user', sa.Integer(), nullable=False),
        sa.ForeignKeyConstraint(['last_Updated_by_user'], ['User.id'], ),
        sa.PrimaryKeyConstraint('id')
        )
    except:
        print "Table AudioPreference already exists."


def create_table_Configuration():
    try:
        op.create_table('Configuration',
        sa.Column('id', sa.Integer(), nullable=False),
        sa.Column('name', sa.String(length=100), nullable=False),
        sa.Column('active', sa.String(length=1), nullable=False),
        sa.Column('level_indicator', sa.String(length=1), nullable=False),
        sa.Column('mix_indicator', sa.String(length=1), nullable=False),
        sa.Column('sdi_indicator', sa.String(length=1), nullable=False),
        sa.Column('dolby_input_type', sa.String(length=20), nullable=True),
        sa.Column('last_Updated_date', sa.DateTime(), nullable=False),
        sa.Column('last_Updated_by_user', sa.Integer(), nullable=True),
        sa.ForeignKeyConstraint(['last_Updated_by_user'], ['User.id'], ),
        sa.PrimaryKeyConstraint('id')
        )
    except:
        print "Table Configuration already exists."


def create_table_Feature():
    try:
        op.create_table('Feature',
        sa.Column('id', sa.Integer(), nullable=False),
        sa.Column('name', sa.String(length=50), nullable=False),
        sa.Column('eeprom', sa.String(length=50), nullable=False),
        sa.Column('last_Updated_date', sa.DateTime(), nullable=False),
        sa.Column('last_Updated_by_user', sa.Integer(), nullable=False),
        sa.ForeignKeyConstraint(['last_Updated_by_user'], ['User.id'], ),
        sa.PrimaryKeyConstraint('id')
        )
    except:
        print "Table Feature already exists."


def create_table_MailSetup():
    try:
        op.create_table('MailSetup',
        sa.Column('id', sa.Integer(), nullable=False),
        sa.Column('mailer', sa.String(length=50), nullable=False),
        sa.Column('host_name', sa.String(length=50), nullable=False),
        sa.Column('port_number', sa.Integer(), nullable=False),
        sa.Column('encryption', sa.String(length=1), nullable=False),
        sa.Column('authentication', sa.String(length=1), nullable=False),
        sa.Column('user_name', sa.String(length=120), nullable=True),
        sa.Column('password_hash', sa.String(), nullable=False),
        sa.Column('last_Updated_date', sa.DateTime(), nullable=False),
        sa.Column('last_Updated_by_user', sa.Integer(), nullable=False),
        sa.ForeignKeyConstraint(['last_Updated_by_user'], ['User.id'], ),
        sa.PrimaryKeyConstraint('id'),
        sa.UniqueConstraint('user_name')
        )
    except:
        print "Table MailSetup already exists."


def create_table_NetworkSetup():
    try:
        op.create_table('NetworkSetup',
        sa.Column('id', sa.Integer(), nullable=False),
        sa.Column('network_type', sa.String(length=50), nullable=False),
        sa.Column('network_ip_address', sa.String(length=20), nullable=False),
        sa.Column('mask_ip_address', sa.String(length=20), nullable=False),
        sa.Column('gateway_ip_address', sa.String(length=20), nullable=False),
        sa.Column('multicast_ip_address', sa.String(length=20), nullable=False),
        sa.Column('port_number', sa.Integer(), nullable=True),
        sa.Column('last_Updated_date', sa.DateTime(), nullable=False),
        sa.Column('last_Updated_by_user', sa.Integer(), nullable=False),
        sa.ForeignKeyConstraint(['last_Updated_by_user'], ['User.id'], ),
        sa.PrimaryKeyConstraint('id')
        )
    except:
        print "Table NetworkSetup already exists."



def create_table_Product():
    try:
        op.create_table('Product',
        sa.Column('id', sa.Integer(), nullable=False),
        sa.Column('name', sa.String(length=100), nullable=False),
        sa.Column('model', sa.String(length=100), nullable=False),
        sa.Column('serial_number', sa.String(length=50), nullable=True),
        sa.Column('hardware_version', sa.String(length=20), nullable=True),
        sa.Column('system_version', sa.String(length=20), nullable=True),
        sa.Column('channel_number', sa.Integer(), nullable=True),
        sa.Column('screen_number', sa.Integer(), nullable=True),
        sa.Column('passcode', sa.Integer(), nullable=False),
        sa.Column('active', sa.String(length=1), nullable=False),
        sa.Column('last_Updated_date', sa.DateTime(), nullable=False),
        sa.Column('last_Updated_by_user', sa.Integer(), nullable=False),
        sa.ForeignKeyConstraint(['last_Updated_by_user'], ['User.id'], ),
        sa.PrimaryKeyConstraint('id')
        )
    except:
        print "Table Product already exists."


def create_table_Screen():
    try:
        op.create_table('Screen',
        sa.Column('id', sa.Integer(), autoincrement=False, nullable=False),
        sa.Column('name', sa.String(length=100), nullable=False),
        sa.Column('code', sa.String(length=100), nullable=True),
        sa.Column('last_Updated_date', sa.DateTime(), nullable=False),
        sa.Column('last_Updated_by_user', sa.Integer(), nullable=False),
        sa.ForeignKeyConstraint(['last_Updated_by_user'], ['User.id'], ),
        sa.PrimaryKeyConstraint('id')
        )
    except:
        print "Table Screen already exists."


def create_table_Specification():
    try:
        op.create_table('Specification',
        sa.Column('id', sa.Integer(), nullable=False),
        sa.Column('name', sa.String(length=100), nullable=False),
        sa.Column('type', sa.String(length=50), nullable=False),
        sa.Column('last_Updated_date', sa.DateTime(), nullable=False),
        sa.Column('last_Updated_by_user', sa.Integer(), nullable=False),
        sa.ForeignKeyConstraint(['last_Updated_by_user'], ['User.id'], ),
        sa.PrimaryKeyConstraint('id')
        )
    except:
        print "Table Specification already exists."


def create_table_Channel():
    try:
        op.create_table('Channel',
        sa.Column('id', sa.Integer(), nullable=False),
        sa.Column('input_type_id', sa.Integer(), autoincrement=False, nullable=False),
        sa.Column('name', sa.String(length=100), nullable=True),
        sa.Column('name2', sa.String(length=100), nullable=True),
        sa.Column('unit_id', sa.Integer(), nullable=False),
        sa.Column('last_Updated_date', sa.DateTime(), nullable=False),
        sa.Column('last_Updated_by_user', sa.Integer(), nullable=True),
        sa.ForeignKeyConstraint(['input_type_id'], ['Specification.id'], ),
        sa.ForeignKeyConstraint(['last_Updated_by_user'], ['User.id'], ),
        sa.ForeignKeyConstraint(['unit_id'], ['Unit.id'], ),
        sa.PrimaryKeyConstraint('id', 'input_type_id')
        )
    except:
        print "Table Channel already exists."

def create_table_ConfigOption():
    try:
        op.create_table('ConfigOption',
        sa.Column('id', sa.Integer(), nullable=False),
        sa.Column('config_id', sa.Integer(), nullable=False),
        sa.Column('name', sa.String(length=100), nullable=False),
        sa.Column('value', sa.String(length=100), nullable=True),
        sa.Column('active', sa.String(length=1), nullable=False),
        sa.Column('last_Updated_date', sa.DateTime(), nullable=False),
        sa.Column('last_Updated_by_user', sa.Integer(), nullable=False),
        sa.ForeignKeyConstraint(['config_id'], ['Configuration.id'], ),
        sa.ForeignKeyConstraint(['last_Updated_by_user'], ['User.id'], ),
        sa.PrimaryKeyConstraint('id'),
        sa.UniqueConstraint('config_id', 'name', name='config_id_name_uidx')
        )
    except:
        print "Table ConfigOption already exists."


def create_table_ProductFeature():
    try:
        op.create_table('ProductFeature',
        sa.Column('product_id', sa.Integer(), autoincrement=False, nullable=False),
        sa.Column('feature_id', sa.Integer(), autoincrement=False, nullable=False),
        sa.Column('license_key', sa.String(length=100), nullable=False),
        sa.Column('last_Updated_date', sa.DateTime(), nullable=False),
        sa.Column('last_Updated_by_user', sa.Integer(), nullable=False),
        sa.ForeignKeyConstraint(['feature_id'], ['Feature.id'], ),
        sa.ForeignKeyConstraint(['last_Updated_by_user'], ['User.id'], ),
        sa.ForeignKeyConstraint(['product_id'], ['Product.id'], ),
        sa.PrimaryKeyConstraint('product_id', 'feature_id')
        )
    except:
        print "Table ProductFeature already exists."


def create_table_ConfigChannel():
    try:
        op.create_table('ConfigChannel',
        sa.Column('config_id', sa.Integer(), autoincrement=False, nullable=False),
        sa.Column('screen_id', sa.Integer(), autoincrement=False, nullable=True),
        sa.Column('channel_id', sa.Integer(), nullable=True),
        sa.Column('display_name', sa.String(length=100), nullable=False),
        sa.Column('label_1', sa.String(length=100), nullable=False),
        sa.Column('label_2', sa.String(length=100), nullable=False),
        sa.Column('channel_2', sa.Integer(), nullable=True),
        sa.Column('mix_id', sa.Integer(), nullable=True),
        sa.Column('mix_type_id', sa.Integer(), nullable=True),
        sa.Column('audio_id', sa.Integer(), nullable=True),
        sa.Column('input_type_id', sa.Integer(), nullable=True),
        sa.Column('audio_input_type_id', sa.Integer(), nullable=True),
        sa.Column('volume', sa.Integer(), nullable=True),
        sa.Column('clock_reference', sa.String(length=1), nullable=True),
        sa.Column('cluster_id', sa.Integer(), nullable=True),
        sa.Column('cluster_name', sa.String(length=20), nullable=True),
        sa.Column('cluster_type', sa.Integer(), nullable=True),
        sa.Column('cluster_label', sa.String(length=50), nullable=True),
        sa.Column('last_Updated_date', sa.DateTime(), nullable=False),
        sa.Column('last_Updated_by_user', sa.Integer(), nullable=True),
        sa.ForeignKeyConstraint(['audio_id'], ['Specification.id'], ),
        sa.ForeignKeyConstraint(['channel_id', 'input_type_id'], [u'Channel.id', u'Channel.input_type_id'], ),
        sa.ForeignKeyConstraint(['cluster_type'], ['Specification.id'], ),
        sa.ForeignKeyConstraint(['config_id'], ['Configuration.id'], ),
        sa.ForeignKeyConstraint(['last_Updated_by_user'], ['User.id'], ),
        sa.ForeignKeyConstraint(['mix_id'], ['Specification.id'], ),
        sa.ForeignKeyConstraint(['mix_type_id'], ['Specification.id'], ),
        sa.PrimaryKeyConstraint('config_id', 'screen_id')
        )
    except:
        print "Table ConfigChannel already exists."

"""
def create_table_xxx():
    try:
        pass
    except:
        print "Table xxx already exists."
"""

def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    # Did some hack by using try except as currently db files are being created and removed form other places too
    create_table_GroupUser()
    create_table_Log()
    create_table_MessageController()
    create_table_Software()
    create_table_Unit()
    create_table_User()
    create_table_AudioPreference()
    create_table_Configuration()
    create_table_Feature()
    create_table_MailSetup()
    create_table_NetworkSetup()
    create_table_Product()
    create_table_Screen()
    create_table_Specification()
    create_table_Channel()
    create_table_ConfigOption()
    create_table_ProductFeature()
    create_table_ConfigChannel()











    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_table('ConfigChannel')
    op.drop_table('VideoConfiguration')
    op.drop_table('ProductFeature')
    op.drop_table('Channel')
    op.drop_table('preset')
    op.drop_table('Specification')
    op.drop_table('Screen')
    op.drop_table('Product')
    op.drop_table('Preference')
    op.drop_table('NetworkSetup')
    op.drop_table('MailSetup')
    op.drop_table('Feature')
    op.drop_table('Configuration')
    op.drop_table('AudioPreference')
    op.drop_table('program')
    op.drop_table('User')
    op.drop_table('programgroup')
    op.drop_table('Unit')
    op.drop_table('Software')
    op.drop_table('MessageController')
    op.drop_table('GroupUser')
    # ### end Alembic commands ###
