// HACK: Don't have import or require here, so let's pluck the modal
// dialog from window.ReactModal.

var Modal = null;
if (typeof window !== 'undefined') {
  Modal = window.ReactModal;
}

function _jqAjaxErrorToMsg(kwargs) {
    /**
     * @param kwargs Object containing:
     *   xhr: The jquery.AJAX xhr object
     *   status: the jquery.AJAX status
     *   err: the jquery.AJAX err object
     *   url: The request URL
     *   logger: The logger or undefined for default
     *   messagePrefix: The message prefix string or undefined/null for default value
     * @type {string}
     */
    var logger = kwargs.logger || Logger.get("AJAX");
    // xhr.readyState == 0 means connection error, CORS blocked, etc.
    // xhr.readyState == 4 means HTTP error (can be checked by XMLHttpRequest.status and XMLHttpRequest.statusText)
    // ref http://stackoverflow.com/a/28404728
    var errMsg = kwargs.messagePrefix || "Error: ";
    if (kwargs.xhr.readyState === 4) {
        errMsg += " (HTTP error: " + kwargs.xhr.statusText + ")";
    } else if (kwargs.xhr.readyState === 0) {
        errMsg += " (connection failure)";
    } else {
        errMsg += " (other error)"
    }
    logger.error(errMsg, kwargs.url, kwargs.xhr, kwargs.status, kwargs.err.toString());
    return errMsg;
}


var DeviceBox = React.createClass({
    getInitialState: function () {
        return {
            // Application states: scan, edit, apply
            scan: {
                scanning: false
            },
            devices: [],
            updates: {files: []},
            myNetwork: {},
            lastError: null,
            lastNotice: null,
            hasEdits: false,
            scanningText: "Scanning..."
        };
    },

    /**
     * Get a list of network config edits.
     */
    getUpdateList: function () {
        return this.state.devices.filter(function (element, index) {
            return element.network._edited;
        });
    },

    handleCommitClick: function () {
        // TODO: walk this.state.devices[] and
        // generate a list of edited items to be updated.
        var updateList = this.getUpdateList();
        if (updateList.length) {
            //  POST to appropriate endpoint with
            // the update list.
            $.ajax({
                url: this.props.networkSettingsUpdateUrl,
                dataType: 'json',
                cache: 'false',
                contentType: "application/json",
                type: 'POST',
                data: JSON.stringify(updateList),
                success: function (data) {
                    console.log("success", data);
                    this.setState({lastNotice: "Network configuration changes sent to target devices.."});
                    // FIXME: Notify user that systems will reboot?
                    this.loadDevicesFromServer();  // FIXME: or, remove the target devices from the list?
                }.bind(this),
                error: function (xhr, status, err) {
                    console.log('error', arguments);
                    this.setState({lastError: err});
                }.bind(this)
            });
        }
    },

    handleFirmwareUpdateClick: function () {
    },

    loadDevicesFromServer: function () {
        // Reset state
        this.setState({
            scan: {
                scanning: true,
            },
            lastError: null
        }, function () {
            $.ajax({
                url: this.props.scanUrl,
                dataType: 'json',
                cache: false,
                success: function (data) {
                    this.setState({
                        devices: data.result.devices,
                        updates: data.result.available_updates,
                        myNetwork: data.result.my_network,
                        scan: {
                            scanning: false
                        }
                    });
                }.bind(this),
                error: function (xhr, status, err) {
                    // xhr.readyState == 0 means connection error, CORS blocked, etc.
                    // xhr.readyState == 4 means HTTP error (can be checked by XMLHttpRequest.status and XMLHttpRequest.statusText)
                    // ref http://stackoverflow.com/a/28404728
                    var errMsg = "Request error";
                    if (xhr.readyState === 4) {
                        errMsg += " (HTTP error: " + xhr.statusText + ")";
                    } else if (xhr.readyState === 0) {
                        errMsg += " (connection failure)";
                    } else {
                        errMsg += " (other error)"
                    }
                    console.error(errMsg, this.props.scanUrl, xhr, status, err.toString());

                    this.setState({
                        scan: {
                            scanning: false,
                            // TODO: store actual error codes from xhr.status if possible.
                        },
                        lastError: errMsg
                    });
                }.bind(this)
            });
        });
    },

    logger: Logger.get('DeviceBox'),

    componentDidMount: function () {
        this.loadDevicesFromServer();
        // setInterval(this.loadDevicesFromServer, this.props.pollInterval);
    },

    handleBlink: function (mac) {
        // TODO: hit this.props.blinkUrl
        $.ajax({
            url: this.props.blinkUrl + mac,
            dataType: 'json',
            cache: 'false',
            contentType: "application/json",
            success: function (data) {
                console.log("success", data);
                this.setState({lastNotice: "Blink command sent to " + mac});
            }.bind(this),
            error: function (xhr, status, err) {
                console.log('error', arguments);
                this.setState({lastError: err});
            }.bind(this)
        });

    },

    handleScanButtonClick: function () {
        this.loadDevicesFromServer();
    },

    handleNameChange: function (deviceInfo, name) {
        this.logger.debug('handleNameChange', deviceInfo, name);
        deviceInfo.name = name;
        if (this.updateDeviceInfo(deviceInfo)) {
            // We updated info, all's good here.
        } else {
            // FIXME: Unable to update local device info state... what to do here?
        }
    },
    handleNetworkEditClick: function (network) {
        this.refs.networkConfigDlg.open(network);
    },

    handleNetworkEditApply: function (device) {
        // TODO: record network settings in pending changes list
        console.log('handleNetworkEditApply', device);
        this.updateDeviceInfo(device);
    },

    updateDeviceInfo: function (deviceInfo) {
        // Find matching MAC in this.state.data array.
        var found = this.state.devices.find(function (item, index, array) {
            if (item.mac === deviceInfo.mac) {
                array[index] = deviceInfo;
                return true;
            }
        });
        if (found) {
            this.setState({devices: this.state.devices, hasEdits: true});
        }
    },

    render: function () {
        return (
            <div>
                <ul className="nav nav-tabs" role="tablist">
                    <li className="active"><a href="#tab_devices" data-toggle="tab" role="presentation">Discovery</a></li>
                    <li><a href="#tab_updates" data-toggle="tab" role="presentation">Updates</a></li>
                </ul>
                <div className="tab-content">
                    <div role="tabpanel"
                         id="tab_devices"
                         className="tab-pane fade in active"
                    >
                        <NetworkConfigDialog
                            ref="networkConfigDlg"
                            onApply={this.handleNetworkEditApply}/>

                        {/* Error message alert */}
                        <div className="error-message">
                            {this.state.lastError
                                ? <div className="alert alert-danger" role="alert">
                                <strong>Error</strong> {this.state.lastError}
                            </div>
                                : null }
                        </div>
                        <div className="notice-message">
                            { this.state.lastNotice
                                ? <div className="alert alert-success" role="alert">
                                <strong>Notice</strong> {this.state.lastNotice}
                            </div>
                                : null
                            }
                        </div>

                        {/* Device list */}
                        <div className="deviceBox container-fluid">
                            { this.state.scan.scanning
                                ? null
                                : <DeviceList
                                onNetworkEdit={this.handleNetworkEditClick}
                                devices={this.state.devices}
                                onBlink={this.handleBlink}
                                onNameChange={this.handleNameChange}
                                myNetwork={this.state.myNetwork}
                            />
                            }
                        </div>

                        {/* Conditional controls */}

                        { this.state.scan.scanning
                            ? <div id="scan-progress">
                            <div className="lead text-center">
                                {this.state.scanningText}
                            </div>
                            <div>
                                <ScanningSpinner />
                            </div>
                        </div>
                            : null
                        }

                        {/* Various controls at bottom of discovered device list.*/}
                        {!this.state.scan.scanning ?
                            <div id="discovery-controls" className="bottom-controls">
                                <button type="button"
                                        className="btn btn-lg btn-primary"
                                        onClick={this.handleScanButtonClick}
                                        disabled={this.state.scan.scanning}>
                                    Scan
                                </button>

                            </div>
                            : null }
                    </div>

                    <div role="tabpanel"
                         className="tab-pane fade"
                         id="tab_updates">
                        <div className="container-fluid">
                            <h3>Firmware Updates</h3>
                            <div className="alert alert-info" role="alert">
                                Select an available firmware update file, then select target devices.
                            </div>
                            {/* fw updates widget */}
                            <RemoteFwUpdatesBox devices={this.state.devices}
                                                disableTargetFilter={DiscoveryConfig.fwupdate_allow_any_target}
                                                updates={this.state.updates}
                                                myNetworkInfo={this.state.myNetwork}
                                                ref="remoteFwUpdates"/>

                        </div>
                    </div>

                </div>
                {/* .tab-content */}
            </div>
        );
    }
});

var DeviceList = React.createClass({
  propTypes: {
      devices: React.PropTypes.array.isRequired,
      onBlink: React.PropTypes.func.isRequired,
      onNetworkEdit: React.PropTypes.func.isRequired,
      myNetwork: React.PropTypes.object.isRequired,
      onNameChange: React.PropTypes.func.isRequired,
  },
  handleNetworkEditClick: function (device) {
    this.props.onNetworkEdit(device);
  },
  render: function() {
    var deviceNodes = this.props.devices.map(function (device) {
        return (
            <DeviceEntry device={device}
                    key={device.mac}
                    onBlink={this.props.onBlink}
                    onNameChange={this.props.onNameChange}
                    onNetworkEdit={this.props.onNetworkEdit.bind(null, device)}
                    webUiNetwork={this.props.myNetwork}
                    />
            );
    }, this);
    return (
      <div className="device-list">
        <div className="device-list-header row">
          <div className="col-lg-2 col-md-2 col-sm-3 col-xs-4">Name</div>
          <div className="col-lg-2 col-md-2 col-sm-3 col-xs-4">MAC</div>
          <div className="col-lg-1 col-md-2 col-sm-2 col-xs-2">Serial#</div>
          <div className="col-lg-1 col-md-2 hidden-sm hidden-xs">Model</div>
          <div className="col-lg-1 col-sm-1 col-xs-2">FW ver.</div>

          <div className="col-lg-2 col-sm-2 col-xs-3">IP Addr</div>
          <div className="col-lg-1 hidden-sm hidden-md hidden-xs">IP Mask</div>
          <div className="col-lg-1 hidden-sm hidden-md hidden-xs">IP GW</div>
          <div className="col-lg-1 hidden-sm hidden-md hidden-xs">Type</div>
          {DiscoveryConfig.enableBlink ?
             <div className="col-lg-1 col-sm-1 col-xs-1">
              Action
             </div> : null}

        </div>
        { deviceNodes }
      </div>
    );
  }
});

/**
 * DeviceNameEntry -- An element representing a device name field.
 * Should support click-to-edit functionality.
 */
var DeviceNameEntry = React.createClass({
    logger: Logger.get('DeviceNameEntry'),
    /**
     * - apply()
     *   Apply current edits
     */
    apply: function () {
        this.setState({
            busy: true
        }, function () {
            /* Update device info */
            var url = 'api/v1/set_unit_name';
            $.ajax({
                url: url,
                cache: false,
                type: 'POST',
                contentType: 'application/json',
                data: JSON.stringify({mac: this.props.device.mac, name: this.state.editName}),
                dataType: 'json',
                success: function (data) {
                    this.logger.debug('XHR success, updating local state.')
                    this.setState({
                        isEditing: false,
                    }, function () {
                        // Update device name in container if value changed.
                        if (this.state.editName !== this.props.device.name) {
                            this.props.onApply(this.props.device, this.state.editName);
                        }
                    });
                }.bind(this),
                error: function (xhr, status, err) {
                    var errMsg = _jqAjaxErrorToMsg({
                        logger: this.logger,
                        xhr: xhr,
                        status: status,
                        err: err,
                        url: url
                    });
                    this.setState({lastError: errMsg});
                }.bind(this),
                complete: function () {
                    this.setState({busy: false});
                }.bind(this)
            });
        });
    },
    cancel: function () {
            this.setState({
                editName: this.props.device.name,
                isEditing: false,
                lastError: ''
            });
    },
    getInitialState: function () {
        return {
            editName: this.props.device.name,
            isEditing: false,
            lastError: '',
            busy: false,
        };
    },
    /**
     * - handleBlur()
     *   When we move off this field, discard changes and exit edit mode.
     * @param e
     */
    handleBlur: function (e) {
        this.apply();
    },
    handleChange: function (event) {
        this.setState({
            editName: event.target.value
        });
    },
    handleFocus: function (e) {
        var target = e.target;
        setTimeout(function() {
            target.select();
        }, 0);
    },
    handleKeyDown: function (e) {
        if (e.keyCode == 27) {
            this.cancel();
        }
    },
    handleSubmit: function (event) {
        event.stopPropagation();
        event.nativeEvent.stopImmediatePropagation();
        this.apply();
        return false;
    },
    onClick: function () {
        this.setState({
            isEditing: !this.state.isEditing
        }, function () {
            React.findDOMNode(this.refs.nameInput).focus();
            this.logger.debug("Click, isEditing=", this.state.isEditing);
        });
    },
    render: function () {
        return (<div className="device device-name col-lg-2 col-md-2 col-sm-3 col-xs-4">
                { this.state.isEditing ?

                    <form className="form-inline" action="#" onSubmit={this.handleSubmit}>
                        <div className={"form-group" + (this.state.lastError ? " has-error" : "")}>
                            { this.state.lastError ?
                                <label className="control-label" htmlFor={this.refs.nameInput}>Unable to set name: {this.state.lastError}</label> : null
                            }
                            <div className="input-group">
                                <input ref="nameInput"
                                       size="20"
                                       maxLength="20"
                                       type="text"
                                       value={this.state.editName}
                                       onChange={this.handleChange}
                                       onFocus={this.handleFocus}
                                       onKeyDown={this.handleKeyDown}
                                       onBlur={this.handleBlur}
                                       placeholder="Name"
                                       className="input-name form-control input-sm"></input>
                                {this.state.busy ? "Wait..." : ""}
                            </div>
                        </div>
                    </form>
                    :
                    <div >
                        {this.props.device.name}
                    </div>
            }</div>
        );
    }
});

var DeviceEntry = React.createClass({
    propTypes: {
        //network: React.PropTypes.object.isRequired,
        webUiNetwork: React.PropTypes.object.isRequired,
        device: React.PropTypes.object.isRequired,
        onNameChange: React.PropTypes.func.isRequired,
    },
  logger: Logger.get('DeviceEntry'),
  util: new Util(),
  render: function () {
    var classNameExtra = this.props.device.network._edited ? ' edited' : '';
      var deviceMacEntry = null;
    if (this.props.device.self) {
        classNameExtra += " device-is-self";
        deviceMacEntry = this.props.device.mac;
    } else  {
        var target = ''; // '_wohler_device_' + this.props.device.mac;
        deviceMacEntry = (<a title="Click to open admin UI" href={"http://" + this.props.device.network.ipaddr}
                     target={target}
                     >
                {this.props.device.mac}
                </a>);
    }
    //console.log("Device.render()", classNameExtra, this.props);
    return (
      <div className={"device device-row row "  + classNameExtra}
        data-mac={this.props.device.mac} >
        <DeviceNameEntry onApply={this.props.onNameChange} device={this.props.device}/>
        <div className="device device-mac col-lg-2 col-md-2 col-sm-3 col-xs-4">
            {deviceMacEntry}
        </div>
        <div className="device device-sn col-lg-1 col-md-2 col-sm-2 col-xs-2">
          {this.props.device.serial}
        </div>
        <div className="device device-model col-lg-1 col-md-2 col-sm-1 hidden-sm hidden-xs">
          {this.props.device.model}
        </div>
        <div className="device device-fw-version col-lg-1 col-sm-1 col-xs-2">
          {this.props.device.firmware.version}
        </div>
        <DeviceNetworkEntry
            data-mac={this.props.device.mac}
            allowEdit={!this.props.device.self}
            isSelf={this.props.device.self}
            network={this.props.device.network}
            onNetworkEdit={this.props.onNetworkEdit}
            />
        {DiscoveryConfig.enableBlink ?
        <div className="device-action col-lg-1 col-sm-1 col-xs-1">
            <div className="btn-group btn-group-xs">
              <button onClick={this.props.onBlink.bind(null, this.props.device.mac)}
                className="btn"><span className="glyphicon glyphicon-screenshot">BLINK</span>
              </button>
                <DeviceFirmwareUpdateNavWidget
                    device={this.props.device}
                />
            </div>
        </div> : null}
      </div>
    );
  }
});

var DeviceFirmwareUpdateNavWidget = React.createClass({
    render: function () {
        return (

                <div className="btn-group btn-group-xs">
                    <a href={"/fwupdate/remote-update/" + this.props.device.mac } className="btn btn-default" aria-label="Left Align">
                      <span className="glyphicon glyphicon-flash" aria-hidden="true">FW</span>
                    </a>
                </div>

        );
    }
});

var DeviceNetworkEntry = React.createClass({
    handleEditClick: function() {
        this.props.onNetworkEdit(this.props.network);
    },
    render: function () {
        return (
            <div className="device-network-info">
                <div className="device device-network-info device-network-addr col-lg-2 col-sm-2 col-xs-3">
                   {this.props.network.ipaddr}
                </div>

                <div className="device device-network-info device-network-mask col-lg-1 col-sm-1 hidden-sm hidden-md hidden-xs">
                  {this.props.network.mask}
                </div>

                <div className="device device-network-info device-network-gw col-lg-1 col-sm-1 hidden-sm hidden-md hidden-xs">
                  {this.props.network.gw}
                </div>

                <div className="device device-network-info device-network-dhcp col-lg-1 col-sm-1 hidden-sm hidden-md hidden-xs">
                  {this.props.network.dhcp ? "DHCP" : "Static"}
                </div>

            </div>
        )
    }
});

var ScanningSpinner = React.createClass({
  render: function () {
    return (
        <div className="scanningSpinner">
          <div className="cssload-loader"></div>
        </div>
    )
  }
});

/** Network Config Edit dialog box and related utilities **/
/**
  * Check an IP address string's format.
  * :param ipaddr: a trimmed string containing the format to check.
  * :return: true if format is correct.
  */

function _checkIpAddrFormat(ipaddr) {
    var patt = /^(?:(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.){3}(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)$/;
    var match = ipaddr.match(patt);
    return ipaddr.match(patt) !== null;
}


function IPnumber(IPaddress) {
    var ip = IPaddress.match(/^(\d+)\.(\d+)\.(\d+)\.(\d+)$/);
    if(ip) {
        return (+ip[1]<<24) + (+ip[2]<<16) + (+ip[3]<<8) + (+ip[4]);
    }
    // else ... ?
    return null;
}


function IPmask(maskSize) {
    return -1<<(32-maskSize)
}

const customStyles = {
  overlay : {
    position          : 'fixed',
    top               : 0,
    left              : 0,
    right             : 0,
    bottom            : 0,
    backgroundColor   : 'rgba(0,0,0, 0.5)'
  },
  content : {
    top                   : '30%',
    left                  : '50%',
    right                 : 'auto',
    bottom                : 'auto',
    marginRight           : '-50%',
    transform             : 'translate(-50%, -50%)',
    backgroundColor       : 'rgba(0,0,0, 0.5)'
  }
};

/**
 * IP / Netmask / Gateway address form field
 *
 **/
var IpAddressFormField = React.createClass({
    getInitialState: function () {
        return {
            value: this.props.defaultValue,
            isValid: true
        };
    },
    propTypes: {
        value:      React.PropTypes.string,
        validator:  React.PropTypes.func,
        onChange:   React.PropTypes.func,
        onBlur:     React.PropTypes.func,
        labelClass: React.PropTypes.string,
        labelText:  React.PropTypes.string.isRequired,
        fieldContainerClass: React.PropTypes.string,
        name:       React.PropTypes.string.isRequired,
        placeholder:React.PropTypes.string,
        disabled:   React.PropTypes.bool,
        invalidText: React.PropTypes.string
    },
    handleBlur: function (event) {
        this.props.onBlur ? this.props.onBlur.apply(this, event) : null;
    },
    handleChange: function (event) {
        this.setState({
                value: event.target.value
            }, function () {
            // this.setState(value) applied, now validate
            this.setState({
                isValid:_checkIpAddrFormat(this.state.value.trim())
            }, function () {
                // this.setState(isValid) complete, notify handler
                this.props.onChange ? this.props.onChange.apply(this, event) : null;
            });
        });
    },
    render: function () {
        return (
            <div className={"form-group " + (this.state.isValid ? '' : 'has-error')}>
                <label className={"form-control-static " + this.props.labelClass}
                    htmlFor={this.props.name}>
                        {this.props.labelText}
                    </label>

                <div className={"input-group " + this.props.fieldContainerClass}>
                    {/* if we want an additional field error indicator here -- but it will require additional layout fixes. */}
                    <span className="inline-help">
                        { this.state.isValid ? null
                            : <span className="help-inline">
                            {this.props.invalidText}
                            </span>
                        }
                    </span>
                    <input
                        className={"form-control " + (this.state.isValid ? '' : 'error')}
                        onBlur={this.handleBlur}
                        onChange={this.handleChange}
                        name={this.props.name}
                        type="text"
                        defaultValue={this.props.defaultValue}
                        placeholder={this.props.placeholder}
                        disabled={this.props.disabled} />

                </div>
            </div>
        )
    }
});

var NetworkConfigDialog = React.createClass({
    propTypes: {
        onApply:   React.PropTypes.func
    },
    getInitialState: function () {
        return {
            isOpen: false,
            invalidFields: [],
            ipFields: [
                {
                    }
            ],
            deviceInfo: {
                mac: '',
                model: '',
                network: {
                    ipaddr: '',
                    dhcp: false,
                    gw: '',
                    mask: '',
                    dns_server: ''
                }
            }
        }
    },
    /**
      * Return a list of field refs having .state.isValid == false.
      */
    getInvalidFields: function () {
        var invalidFields = Object.keys(this.refs).filter(function (key) {
            // Only check refs with .state.isValid properties.
            var f = this.refs[key];
            if (f.state && f.state.hasOwnProperty('isValid')) {
                return !f.state.isValid;
            };
            return false;
        }, this);
        return invalidFields;
    },
    extractFieldsToNetworkObj: function () {
        var network = {
            'dhcp': this.refs.fieldDhcp.checked,
            'ipaddr': this.refs.fieldIp.state.value,
            'gw': this.refs.fieldGateway.state.value,
            'mask': this.refs.fieldMask.state.value,
            'dns_server': this.refs.fieldDnsServer.state.value,
            '_edited': true
        };
        return network;
    },
    handleApply: function () {
        var newDeviceInfo = this.state.deviceInfo,
            network = this.extractFieldsToNetworkObj();
        newDeviceInfo.network = network;
        this.setState({deviceInfo: newDeviceInfo}, function () {
            this.props.onApply(this.state.deviceInfo);
        });
    },
    handleClose: function (save) {
        if (save) {
            var newConfig = this.extractFieldsToNetworkObj();
            this.handleApply();
        }
        this.setState({isOpen:false});
    },
    handleDhcpChange: function (event) {
        "use strict";
        // Update only the deviceInfo.network.dhcp field (merge)
        var tDevice = this.state.deviceInfo;
        tDevice.network.dhcp = event.target.checked;
        this.setState({
            deviceInfo: tDevice
        }, function () {
            // TODO: enable/disable form fields depending on DHCP state
        });
    },
    handleFieldChanged: function () {
        this.setState({invalidFields: this.getInvalidFields()});
    },
    open: function (deviceInfo) {
        this.setState({
            deviceInfo: deviceInfo,
            invalidFields: []   // Force reset invalid fields list
        }, function () {
            // Wait for DOM updates from deviceInfo setState call.
            this.setState({isOpen:true}, function () {
            });
        });
    },
    render: function () {
        return (
            <Modal
              className="modal-dialog"
              isOpen={this.state.isOpen}
              onRequestClose={this.handleClose}
              style={customStyles} >
              <div className="modal-content">
                <div className="modal-header">
                  <button type="button" className="close" onClick={this.handleClose.bind(null, false)}>
                    <span aria-hidden="true">&times;</span>
                    <span className="sr-only">Close</span>
                  </button>
                  <h4 className="modal-title">Network settings for {this.state.deviceInfo.model} ({this.state.deviceInfo.mac})</h4>
                </div>
                <form>
                <div className="container-fluid">
                    <div className="modal-body form-horizontal">
                        <div className="form-group">
                            <label className="form-control-static col-sm-3" htmlFor="dhcp">
                                Use DHCP?
                            </label>
                            <div className="col-sm-8">
                                <div className="col-sm-2">
                                <input className="col-sm-2 form-control" ref="fieldDhcp"
                                    onChange={this.handleDhcpChange}
                                        name="dhcp"
                                        type="checkbox"
                                        checked={this.state.deviceInfo.network.dhcp}>
                                </input>
                                </div>
                            </div>
                        </div>
                        {/* TODO: We can use a loop/map for this (http://stackoverflow.com/a/22877110) */}
                        {/* Variant values: refName, name, label, defaultValue, placeholder */}
                        <div id="nonDhcpFields">
                            <IpAddressFormField ref="fieldIp"
                                name="ip"
                                labelText="IP"
                                labelClass="col-sm-3"
                                fieldContainerClass="col-sm-8"
                                defaultValue={this.state.deviceInfo.network.ipaddr}
                                placeholder="IP Address"
                                disabled={this.state.deviceInfo.network.dhcp}
                                onChange={this.handleFieldChanged}
                                />

                            <IpAddressFormField ref="fieldMask"
                                name="mask"
                                labelText="Mask"
                                labelClass="col-sm-3"
                                fieldContainerClass="col-sm-8"
                                defaultValue={this.state.deviceInfo.network.mask}
                                placeholder="Network mask"
                                disabled={this.state.deviceInfo.network.dhcp}
                                onChange={this.handleFieldChanged}
                                />

                            <IpAddressFormField ref="fieldGateway"
                                name="gw"
                                labelText="Gateway"
                                labelClass="col-sm-3"
                                fieldContainerClass="col-sm-8"
                                defaultValue={this.state.deviceInfo.network.gw}
                                placeholder="Gateway IP address"
                                disabled={this.state.deviceInfo.network.dhcp}
                                onChange={this.handleFieldChanged}
                                />

                            <IpAddressFormField ref="fieldDnsServer"
                                name="dnsServer"
                                labelText="DNS Server"
                                labelClass="col-sm-3"
                                fieldContainerClass="col-sm-8"
                                defaultValue={this.state.deviceInfo.network.dns_server}
                                placeholder="DNS Server IP address"
                                disabled={this.state.deviceInfo.network.dhcp}
                                onChange={this.handleFieldChanged}
                                />
                        </div>
                    </div>
                </div>
                <div className="modal-footer">
                  <button type="button"
                    disabled={this.state.invalidFields.length}
                    className="btn btn-default"
                    onClick={this.handleClose.bind(null, true)}>
                      OK
                    </button>
                  <button type="button" className="btn btn-primary" onClick={this.handleClose.bind(null, false)}>Cancel</button>
                </div>
                <div className="modal-errors">
                  {this.state.invalidFields.length
                    ? <div className="alert alert-danger" role="alert">
                        <strong>Validation failed.</strong>  Please check and correct the form values.
                      </div>
                    : null}
                </div>
              </form>
              </div>

            </Modal>
        )
    }
});

var deviceBox = ReactDOM.render(
    <div>
        <DeviceBox
            scanUrl={DiscoveryConfig.urls.discovery}
            networkSettingsUpdateUrl={DiscoveryConfig.urls.networkSettingsUpdate}
            blinkUrl={DiscoveryConfig.urls.blink} />
    </div>,
    document.getElementById('deviceBox')
);

//var updatesbox = ReactDOM.render(
//    <div>
//        <FirmwareUpdatesWidget url={DiscoveryConfig.urls.fwGetUpdates}/>
//    </div>,
//    document.getElementById('updatesBox')
//);
