var Modal = null;
if (typeof window !== 'undefined') {
  Modal = window.ReactModal;
}

var RemoteFwUpdatesBox = React.createClass({
    mixins: [OnUnload],
    logger: Logger.get('RemoteFwUpdatesBox'),

    onBeforeUnload: function (e) {
        e = e || window.event;
        if (this.state.processState !== 'idle') {
            e.preventDefault = true;
            e.cancelBubble = true;
            var msg = 'Remote firmware updates are in progress.'
                + '  Leaving this page will stop all pending updates, and you'
                + ' will lose any selections made on this page.';
            e.returnValue = msg;
            return msg;
        }
    },

    propTypes: {
        disableTargetFilter: React.PropTypes.bool,
        updates: React.PropTypes.object.isRequired,
        devices: React.PropTypes.array.isRequired,
        myNetworkInfo: React.PropTypes.shape({
            ipaddr: React.PropTypes.string,
            mask: React.PropTypes.string,
            prefix_length: React.PropTypes.number
        }).isRequired
    },
    getInitialState: function () {
        return ({
            selectedFile: null,
            targets: [], /* A simple list of target objects */
            processState: 'idle',
            upgradeState: 0,
            progress: {
                currentTarget: null,
                currentStatus: null,
                currentItemNumber: 1
            },
            isValidFile: false,
            uploadText: ' + Add File ...'
        })
    },


    getStatusTextForStatusCode: function(statusCode) {
        var map = {
            'push-begin'        : 'sending file to target',
            'push-end'          : 'file sent to target',
            'apply-request'     : 'apply update',
            'apply-end'         : 'update applied',
            'operation-success' : 'operation succeeded',
            'operation-error'   : 'operation failed'
        };
        var statusText = (statusCode in map) ? map[statusCode] : statusCode;
        //this.logger.debug('status text for ' + statusCode + '=' + statusText);
        return statusText;
    },

    __applyUpdates: function () {
        if (confirm('Are you sure you want to proceed?')) {
            var currentItemNumber = 1;
            // set this.state.processState != 'idle' to signal activity.
            this.setState({
                processState: 'running'
            }, function () {
                var self = this;
                this.refs.progressDlg.open(this.state.selectedFile, this.state.targets);
                var updater = new FwUpdater({
                    cb: {
                        status: function(p) {
                            var msg = 'target='
                                + p.target
                                + ': '
                                + self.getStatusTextForStatusCode(p.status); // "update status: " + p.target + ': status=' + p.status;
                            self.logger.debug(msg, p);
                            // Update current progress state
                            if (p.status === 'operation-success' || p.status === 'operation-error') {
                                currentItemNumber++;
                                self.logger.debug('currentItemNumber', currentItemNumber);
                            }
                            self.setState({
                                progress: {
                                    currentTarget: p.target,
                                    currentStatus: self.getStatusTextForStatusCode(p.status),
                                    currentItemNumber: currentItemNumber
                                }
                            }, function () {
                                // Add line to log panel after state updated.
                                self.refs.progressDlg && self.refs.progressDlg.appendLogLine(msg);
                            });
                        }
                    }
                });

                // Get an array of hostnames/ip address targets.
                var targetHostNames = this.state.targets.map(function(target) {
                    return target.network.ipaddr;
                });

                this.logger.debug('applyUpdates: targetHostnames', targetHostNames);

                if (targetHostNames.length > 0) {
                    // We have at least one target, so let's go for it.
                    updater.updateTargets(targetHostNames, this.state.selectedFile.filename, this.state.selectedFile.source).then(
                        function(status) {
                            /**
                             * Process complete, set idle state
                             */
                            self.logger.debug('updateTargets resolved, updates completed.  status=', status);
                            self.refs.progressDlg
                                && self.refs.progressDlg.appendLogLine('Update processing complete.  All updates applied.');
                            self.setState({
                                processState: 'idle',
                                progress: {
                                    currentTarget: '',
                                    currentStatus: 'Done',
                                    currentItemNumber: 0
                                }
                            });
                        }
                    );
                } else {
                    this.logger.warn('no targets found?  targetHostNames=', targetHostNames, 'this.state.targets=', this.state.targets);
                    /**
                     * Process complete, set idle state
                     */
                    self.refs.progressDlg && self.refs.progressDlg.appendLogLine('No targets selected?  Nothing to do.');
                    this.setState({
                        processState: 'idle'
                    })

                }
            });
        }
    },

    __changeFileSelection: function(file, e) {
        this.setState({
            selectedFile: file
        });
    },

    __handleProgressDialogClose: function () {
        // Navigate back to discovery now
        window.location.replace('/discovery');
    },

    __shouldDisableApplyButton: function () {

        var hasFile = this.state.selectedFile !== null,
            hasTargets = false;

        if (this.refs.targetDevices) {
            var td = this.refs.targetDevices.state.selectedTargets;

            // Test for empty object (hash)
            // http://stackoverflow.com/a/32108184/1309774
            hasTargets = !(Object.keys(td).length === 0 && JSON.stringify(td) === JSON.stringify({}));
        }
        var emptyFilesOrTargets = !(hasFile && hasTargets);
         // console.log ("emptyFilesOrTargets");
         // console.log (emptyFilesOrTargets);
        if (emptyFilesOrTargets) { return true }


        var packageFile = this.state.selectedFile;
        // console.log("packageFile");
        // console.log(packageFile);
        var hasInvalidTargetUnits = false;
        if (this.refs.targetDevices &&  packageFile !== null ){

            var packageFileVersion = packageFile['metadata']['version_float'];
            // console.log("packageFileVersion");
            // console.log(packageFileVersion);
            // console.log(packageFile);
            var Units =  this.refs.targetDevices.state.selectedTargets;
            var targetUnits = Object.keys(Units).map(function(key){
                                                                    return Units[key];
                                                                  });
            for ( var idx=0; idx < targetUnits.length; idx++){
                var targetVersion = targetUnits[idx]['firmware']['version_float'];
                // console.log("targetVersion");
                // console.log(targetVersion);
                if (targetVersion > packageFileVersion){
                   hasInvalidTargetUnits = true;
                   idx = targetUnits.length;
                }
            }
        }

        // console.log("hasInvalidTargetUnits");
        // console.log(hasInvalidTargetUnits);
        return hasInvalidTargetUnits;

        //if (hasInvalidTargetUnits){ console.log("hasValidTargetUnits"); return true; }
        // var hasFile = this.state.selectedFile !== null,
        //     hasTargets = false;
        // var isUpgrade = this.state.upgradeState;
        // if (isUpgrade === 1){ console.log("isUpgade == 1"); return true; }



        // if (this.refs.targetDevices) {
        //     var td = this.refs.targetDevices.state.selectedTargets;
        //
        //     // Test for empty object (hash)
        //     // http://stackoverflow.com/a/32108184/1309774
        //     hasTargets = !(Object.keys(td).length === 0 && JSON.stringify(td) === JSON.stringify({}));
        // }
        // return !(hasFile && hasTargets);
    },

    __targetSelectionChanged: function (selectedTargetsObj) {
        /**
         * Selected target changes handler.
         * Side effect: mutates this.state.targets.
         * :param selectedTargetsObj: An object map containing mac address->device info objects.
         *
         **/

        // var isValidUpdate = true;
        // var src_version = 0;
        // if (this.state.selectedFile) {
        //     src_version = this.state.selectedFile['metadata']['version_float']
        // }
        //
        // var newTargets = [];
        var newTargets = Object.keys(selectedTargetsObj).map(function(key){
                 return selectedTargetsObj[key];
            });


        // if (src_version === 0) {
        //     newTargets = Object.keys(selectedTargetsObj).map(function(key){
        //         return selectedTargetsObj[key];
        //     });
        // }
        // else{
        //     newTargets = Object.keys(selectedTargetsObj).filter(function(key){
        //             return selectedTargetsObj[key]['firmware']['version_float'] <= src_version ;
        //         }).map( function(key){return selectedTargetsObj[key]});
        // }


        // console.log("newTargets");
        // console.log(newTargets);

        this.logger.debug('newTargets', newTargets);
        this.setState({
            targets: newTargets
        }, function () {
            this.forceUpdate();
        });






        // var newTargets  = [];
        // var oldTargets = Object.keys(selectedTargetsObj).map(function (key) {
        //     return selectedTargetsObj[key];
        // });
        //
        // for (var idx = 0; idx < oldTargets.length; idx++) {
        //     var target_version = oldTargets[idx]['firmware']['version_float'];
        //     if ( this.state.selectedFile && (target_version > src_version)){
        //         isValidUpdate = false;
        //         idx = oldTargets.length;
        //     }
        // }
        //
        // if (isValidUpdate) {
        //    //console.log("valid");
        //    newTargets = Object.keys(selectedTargetsObj).map(function(key){
        //        return selectedTargetsObj[key];
        //    });
        //    this.logger.debug('newTargets', newTargets);
        //
        //    if (this.state.selectedFile !== null) {
        //        //console.log("File != null");
        //        this.setState({
        //            targets: newTargets,
        //            upgradeState: 0
        //        }, function () {
        //            this.forceUpdate();
        //        });
        //    }
        //    else{
        //         //console.log("File == null");
        //         this.setState({
        //            targets: newTargets,
        //            upgradeState: 1
        //        }, function () {
        //            this.forceUpdate();
        //        });
        //     }
        // }
        // else {
        //     //console.log("invalid");
        //     this.setState({
        //        upgradeState: 1,
        //    }, function () {
        //         this.forceUpdate();
        //    });
        //    this.__shouldDisableApplyButton();
        // }

    },

    __uploadFileToUnit: function(files){

        let  url = '/fwupdate/uploadfile';
        let form_data = new FormData();
        form_data.append('filename', files[0]['name']);
        form_data.append('file', files[0]);
        let success_upload = false;
        show_modal_message(true);
        $.ajax(url, {
            method: 'POST',
            type: 'POST',
            enctype: 'multipart/form-data',
            data: form_data,
            contentType: false,
            cache: false,
            processData: false,
            async: true,
            success: function (data) {
                show_modal_message(false);
                location.reload();
            },
            error: function (error) {
                console.log(error);
            }
        });

    },

    isOnMySubnet: function (address) {
        try {
            var myAddr = ipaddr.parse(this.props.myNetworkInfo.ipaddr),
                otherAddr = ipaddr.parse(address);
            return myAddr.match(otherAddr, this.props.myNetworkInfo.prefix_length);
        } catch (ex) {
            this.logger.debug("isOnMySubnet failed", ex);
            return false;
        }
    },

    render: function () {
       var updatePanelContent,
           filteredDevices = this.props.devices.filter(function(d) {
               /* Don't include self, or those units not on my subnet */
               return this.props.disableTargetFilter
                   || (!d.self && this.isOnMySubnet(d.network.ipaddr, d.network.mask));
            }, this);

       const FileSelector = (props:{onLoadFile: (files:FileList)=> void}) => (
                <input type="file" accept=".deb" onChange={(e:React.ChangeEvent<HTMLInputElement>) => props.onLoadFile(e.target.files)} />);

       if (this.props.updates.files.length) {
           updatePanelContent = this.props.updates.files.map(function (f) {
               return (

                   <div className="radio" key={f.filename}>
                       <input type="radio"
                              onChange={this.__changeFileSelection.bind(this, f)}
                              name="availableUpdates"
                              value={f.filename}
                              id={"file-" + f.filename}>
                              </input>
                       <label htmlFor={"file-" + f.filename}>{f.filename} (v{f.metadata.version})</label><span className="label file-src">{f.source === 'NET' ?'Added file' : 'USB' }</span>
                   </div>
               );
           }, this);
       } else {
           updatePanelContent = (
               <div className="alert alert-warning" role="alert">
                   <p>USB flash drive contains no Wohler Update Packages.</p>
               <p>Insert a USB flash drive containing Wohler Update Packages and
                   <a className="btn btn-default btn-xs" href="/discovery">re-scan</a>.</p>
               </div>
           )
       }
       return (
           <div id="remoteFwUpdates" className="container-fluid">
               <RemoteUpdatesProgressDialog
                   file={this.state.selectedFile}
                   progress={this.state.progress}
                   runState={this.state.processState}
                   totalItems={this.state.targets.length}
                   onClose={this.__handleProgressDialogClose}
                   ref="progressDlg"/>
               <div className="row">
                   <div className="col-md-4">
                       <div className="panel-group">
                           <div className="panel panel-default">
                               <div><br/><br/></div>
                               <div className="panel-heading">
                                   <h3 className="panel-title">Available updates</h3>
                               </div>
                               <div className="panel-body" id="fwUpdatesFiles">
                                   {updatePanelContent}
                               </div>
                           </div> {/* inner.panel */}

                           <br />

                           <div className="panel panel-default">
                               <div className="panel-heading">
                                   <h3 className="panel-title">Upload a single file</h3>
                               </div>

                               <div className="panel-body" id="fwUpdatesFilesNet">
                                   <div className="table-responsive">

                                       <table className="table">
                                           <tbody>
                                           <tr>
                                               <td>
                                                    <label className="btn btn-primary btn-file"  >
                                                        {this.state.uploadText}
                                                    <FileSelector onLoadFile={(files:FileList) => {
                                                               let uploadedFile = files[0]['name'];
                                                               if (uploadedFile.endsWith(".deb")  && uploadedFile.startsWith('iam-audio_')){
                                                                   this.isValidFile = true;
                                                                   this.__uploadFileToUnit(files);
                                                               }
                                                               else {
                                                                   console.log("Invalid file");
                                                                }
                                                            }
                                                        }

                                                    />
                                                    </label>
                                               </td>
                                               <td>

                                               </td>
                                           </tr>

                                           </tbody>
                                       </table>


                                   </div>

                               </div>

                            </div> {/* inner.panel */}



                       </div>
                   </div> {/* .col-md-4 */}


                   <div className="col-md-8">
                       <div className="panel panel-default">
                           <div className="panel-heading">
                                <h3 className="panel-title">Target devices</h3>
                           </div>
                           <div className="panel-body">
                               { filteredDevices.length ?
                                   <TargetDevicesSelector
                                       ref="targetDevices"
                                       devices={filteredDevices}
                                       selectionChangeHandler={this.__targetSelectionChanged} />
                               :  <div className="alert alert-warning">
                                   No eligible target devices found on this network subnet.
                                     Return to discovery panel and <a className="btn btn-default btn-xs" href="/discovery">re-scan</a>.
                                   </div>}
                           </div> {/* .panel-body */}
                       </div>
                   </div>
               </div> {/* .row */}

               <div className="row bottom-controls">
                   <button className="btn btn-lg btn-primary"
                           onClick={this.__applyUpdates}
                           disabled={this.__shouldDisableApplyButton()}
                   >
                       Apply selected updates
                   </button>
               </div>
           </div>
       )
   }
});


var TargetDevicesSelector = React.createClass({
    getDefaultProps: function () {
        return {
            devices: []
        }
    },
    getInitialState: function () {
        /* selectedTargets is a map of mac addresses to target devices */
        return {
            selectedTargets: {}
        }
    },
    propTypes: {
        devices: React.PropTypes.array
    },

    __changeSelection: function (device, e) {
        /* A device selection has changed. Update internal selection
           state.

           :param device: The device selected/deselected.
           :param e: The selection event.
         */
        var checked = e.target.checked,
            newTargets = this.state.selectedTargets;
        if (checked) {
            newTargets[device.mac] = device;
        } else {
            delete newTargets[device.mac]
        }
        // Update state now.
        this.setState({
            selectedTargets: newTargets
        }, function () {
            /* Notify of selection change */
            this.props.selectionChangeHandler(newTargets, e);
        });
    },

    render: function () {
        /* Filter out any undesired nodes */
        /* TODO: Should filter once rather than on every render */
        var targetDeviceNodes = this.props.devices.filter(function (d){ return d.model == 'iAM-Audio' && d.firmware['version_float'] > 0;}).map(function (d) {
            return (
                <tr key={d.mac}>
                    <td className="ipaddr">
                        <div className="checkbox">
                            <input type="checkbox"
                                   onChange={this.__changeSelection.bind(this, d)}
                                   name="targetDevice"
                                   id={"targetsel-" + d.mac}
                                   value={d.network.ipaddr}/>
                            <label htmlFor={"targetsel-" + d.mac}>{d.network.ipaddr}</label>
                        </div>
                    </td>
                    <td className="macaddr">{d.mac}</td>
                    <td className="version">{d.firmware.version}</td>
                </tr>
            );
        }, this);
        return (
            <div>
                <table className="table table-striped">
                    <thead>
                    <tr>
                        <td className="ipaddr">IP Address</td>
                        <td className="macaddr">MAC</td>
                        <td className="version">Version</td>
                    </tr>
                    </thead>
                    <tbody>
                        {targetDeviceNodes}
                    </tbody>
                </table>
            </div>
        )
    }
});

// Make components available on the global namespace.
if (typeof window != 'undefined') {
    window.RemoteFwUpdatesBox = RemoteFwUpdatesBox;
}

