"""Init preset and data migration

Revision ID: dcdedd764643
Revises:
Create Date: 2018-01-09 19:08:38.189948

"""
from alembic import op
import sqlalchemy as sa
from datetime import datetime


# revision identifiers, used by Alembic.
revision = 'dcdedd764643'
down_revision = None
branch_labels = None
depends_on = None




def create_table_programgroup():
    try:
        op.create_table('programgroup',
        sa.Column('Id', sa.Integer(), nullable=False),
        sa.Column('FactoryDefault', sa.Integer(), nullable=True),
        sa.Column('Name', sa.String(length=64), nullable=False),
        sa.Column('Data', sa.BLOB(), nullable=True),
        sa.PrimaryKeyConstraint('Id'),
        sa.UniqueConstraint('Name')
        )
    except:
        print "Table programgroup already exists."



def create_table_program():
    try:
        op.create_table('program',
        sa.Column('Id', sa.Integer(), nullable=False),
        sa.Column('GroupId', sa.Integer(), nullable=True),
        sa.Column('FactoryDefault', sa.Integer(), nullable=True),
        sa.Column('Name', sa.String(length=64), nullable=False),
        sa.Column('Data', sa.BLOB(), nullable=True),
        sa.ForeignKeyConstraint(['GroupId'], ['programgroup.Id'], ),
        sa.PrimaryKeyConstraint('Id'),
        sa.UniqueConstraint('GroupId', 'Name', name='_program_uc')
        )
    except:
        print "Table program already exists."



def create_table_preset():
    try:
        op.create_table('preset',
        sa.Column('Id', sa.Integer(), nullable=False),
        sa.Column('ProgramId', sa.Integer(), nullable=True),
        sa.Column('FactoryDefault', sa.Integer(), default=0, nullable=False),
        sa.Column('Name', sa.String(length=64), nullable=False),
        sa.Column('Data', sa.BLOB(), nullable=True),
        sa.Column('last_Updated_date', sa.DateTime(), nullable=False),
        sa.ForeignKeyConstraint(['ProgramId'], ['program.Id'], ),
        sa.PrimaryKeyConstraint('Id'),
        sa.UniqueConstraint('ProgramId', 'Name', name='_preset_uc')
        )
    except:
        print "Table preset already exists."


def upgrade():
    # Did some hack by using try except as currently db files are being created and removed form other places too

    create_table_programgroup()
    create_table_program()
    create_table_preset()

def downgrade():
    op.drop_table('programgroup')
    op.drop_table('program')
    op.drop_table('preset')
