#!/bin/bash    

####################################
# DB Backup : iAM-AUDIO
####################################
#echo $BKP_FILE_NAME
echo "    First  parameter : $1"
echo "    Second parameter : $2"
echo "    Third  parameter : $3"
echo "    Four   parameter : $4"
echo "    Fifth  parameter : $5"
echo "    Sixth  parameter : $6"


BASE_PATH="/tmp/uploaddownload/"
EXPORT_FILE_PATH="$BASE_PATH/a_export"
EXP_FILE_NAME="$3"
TARGET_USB_PATH="/media/usb0/wohler"
IMPORT_FILE_PATH="$BASE_PATH/a_import"

WAUDIO_DB_FILE="wohler_audio.db"
WPRESETS_DB_FILE="wohler_presets.db"

SHM_DB_PATH="/run/shm/database"
SYSAPP_DB_PATH="/root/iam_sysapp/iam_audio_ui/database"
DEST_DB_BKP_PATH="$SYSAPP_DB_PATH/backup"
SHM_DB_BKP_PATH="$SHM_DB_PATH/backup"

# fmtdDate=$(date  +"%Y%m%d%H%M%S")
# CONFIG_DB_NAME="$3"

WOHLER_DB_FILE_BKP="$5"
WOHLER_PRESET_DB_FILE_BKP="$6"


if [ "$1" = "1" ]
then

    ####################################
    # DB backup
    ####################################
    echo "$SHM_DB_BKP_PATH/$WOHLER_DB_FILE_BKP"
    echo "$SHM_DB_BKP_PATH/$WOHLER_PRESET_DB_FILE_BKP"


    if [ -f "$SHM_DB_BKP_PATH/$WOHLER_DB_FILE_BKP" -a "$SHM_DB_BKP_PATH/$WOHLER_PRESET_DB_FILE_BKP" ]
    then
        echo "$WOHLER_DB_FILE_BKP and $WOHLER_PRESET_DB_FILE_BKP were found."
        cd $SHM_DB_BKP_PATH
        tar -czvf $WOHLER_DB_FILE_BKP.tar.gz  $WOHLER_DB_FILE_BKP
        sleep 2
        tar -czvf  $WOHLER_PRESET_DB_FILE_BKP.tar.gz  $WOHLER_PRESET_DB_FILE_BKP

        #
        # Moving files to SD card
        #
        echo "Moving files to SD Card"
        mv $SHM_DB_BKP_PATH/$WOHLER_DB_FILE_BKP.tar.gz $DEST_DB_BKP_PATH/.
        mv $SHM_DB_BKP_PATH/$WOHLER_PRESET_DB_FILE_BKP.tar.gz $DEST_DB_BKP_PATH/.

        echo "Deleting files"
        rm $SHM_DB_BKP_PATH/$WOHLER_DB_FILE_BKP
        rm $SHM_DB_BKP_PATH/$WOHLER_PRESET_DB_FILE_BKP

        echo "Backup has been successfully done."

    else
        echo "wohler_audio.db and wohler_presets.db were not found."
              exit 1
    fi
    exit 0

elif [ "$1" = "2" ]
then
    ####################################
    ## Export Presets
    ####################################

    if [ ! -d "$EXPORT_FILE_PATH" ]
    then
        echo " Export Directory does not exist. "
        exit 2
    fi

    if [ -f "$EXPORT_FILE_PATH/audio_presets.json" ]
    then
        cd $BASE_PATH
        echo " Export Presets : Preparing tar files ... "

        #
        # creating tar files
        #

        tar -czvf $EXP_FILE_NAME a_export

        if [ ! -d "$TARGET_USB_PATH" ]
        then
            mkdir $TARGET_USB_PATH
        fi

        mv $EXP_FILE_NAME $TARGET_USB_PATH/.
        rm -r $BASE_PATH/a_export

    else
        echo " Export Preset files do not exist. "
        exit 2
    fi

    echo " Export has been successfully done."
    exit 0

elif [ "$1" = "3" ]
then

    ####################################
    # DB Restore (dump) option
    ####################################
     
    echo " DB Restore (dump) : Executing command "
    echo "$DEST_DB_BKP_PATH/$WOHLER_DB_FILE_BKP"
    echo "$DEST_DB_BKP_PATH/$WOHLER_PRESET_DB_FILE_BKP"

    if [ -f "$DEST_DB_BKP_PATH/$WOHLER_DB_FILE_BKP.tar.gz" -a "$DEST_DB_BKP_PATH/$WOHLER_PRESET_DB_FILE_BKP.tar.gz" ]
    then

        if [ ! -d "$SHM_DB_BKP_PATH" ]
        then
            mkdir $SHM_DB_BKP_PATH
        fi

        echo "Extracting wohler_audio.db"
        cd  $SHM_DB_BKP_PATH
        tar -zxmf $DEST_DB_BKP_PATH/$WOHLER_DB_FILE_BKP.tar.gz 
        mv $WOHLER_DB_FILE_BKP $WAUDIO_DB_FILE

        echo "Extracting wohler_presets.db"
        cd  $SHM_DB_BKP_PATH
        tar -zxmf $DEST_DB_BKP_PATH/$WOHLER_PRESET_DB_FILE_BKP.tar.gz
        mv $WOHLER_PRESET_DB_FILE_BKP $WPRESETS_DB_FILE
        ls -l $SHM_DB_BKP_PATH
        echo ""
        echo "Restoring wohler_audio.db"
        cp -f $WAUDIO_DB_FILE $SYSAPP_DB_PATH/.
        echo ""
        echo "Restoring wohler_presets.db"
        cp -f $WPRESETS_DB_FILE $SYSAPP_DB_PATH/.
        sync; sync;
        exit 0
    else
        echo "   DB file not found. DO NOT RESTORE"
        exit 3

    fi
    exit 0


elif [ "$1" = "4" ]
then
    ####################################
    # Import Configuration
    ####################################

    echo "    Import"
    echo "$TARGET_USB_PATH/$EXP_FILE_NAME"
    echo ""

    echo "Checking and Removing old data"
        if [ -d "$IMPORT_FILE_PATH" ]
        then
            rm -rf $IMPORT_FILE_PATH
        fi

        if [ -f "$TARGET_USB_PATH/$EXP_FILE_NAME" ]
        then
            echo "Importing : Wohler DB exist. Reading file ..."

            if [ ! -d "$IMPORT_FILE_PATH" ]
            then
                mkdir -p $IMPORT_FILE_PATH
            fi

            if [ -d "$IMPORT_FILE_PATH" ]
            then
                echo "Importing : DB is being copied... "
                cp -p $TARGET_USB_PATH/$EXP_FILE_NAME  $IMPORT_FILE_PATH/.
                cd $IMPORT_FILE_PATH
                pwd
                echo "$IMPORT_FILE_PATH/$EXP_FILE_NAME"

                if file --mime-type "$IMPORT_FILE_PATH/$EXP_FILE_NAME" | grep -q gzip$; then
                    echo "$IMPORT_FILE_PATH/$EXP_FILE_NAME is gzipped file."
                    echo "It seems like the file contains new presets data."
                    tar -xzvf $IMPORT_FILE_PATH/$EXP_FILE_NAME
                    ls -l
                    exit 0
                else
                    echo "$IMPORT_FILE_PATH/$EXP_FILE_NAME is not gzipped file"
                    echo "It seems like the file contains OLD presets data."
                fi

            else
                echo " Directory $SYSAPP_DB_PATH/csv/ does not exist"
                exit 4
            fi

        fi

else
    echo " Invalid Option"
    echo " Usage : db_management.sh action bkp-date"
    echo "         action : 1 = Backup DB File "
    echo "                  2 = Export Configuration wdb File to USB"
    echo "                  3 = Restore from DB File "
    echo "                  4 = Import Configuration wdb File from USB "
    echo "         backup date : YYYYMMDDHHMMSS eg 20150922074736 "
    echo " Example (Restoring ) : sh db_management.sh 3 \"20150922074736\"  "
fi
     


